@@ -25,7 +25,8 @@ extern "C" {
 
 class OpendalBddTest : public ::testing::Test {
 protected:
-    opendal_operator_ptr p;
+    const opendal_operator_ptr *p;
+
     std::string scheme;
     std::string path;
     std::string content;
@@ -36,17 +37,17 @@ protected:
         this->path = std::string("test");
         this->content = std::string("Hello, World!");
 
-        opendal_operator_options options = opendal_operator_options_new();
-        opendal_operator_options_set(&options, "root", "/myroot");
+        opendal_operator_options *options = opendal_operator_options_new();
+        opendal_operator_options_set(options, "root", "/myroot");
 
         // Given A new OpenDAL Blocking Operator
-        this->p = opendal_operator_new(scheme.c_str(), &options);
-        EXPECT_TRUE(this->p.ptr);
+        this->p = opendal_operator_new(scheme.c_str(), options);
+        EXPECT_TRUE(this->p->ptr);
 
-        opendal_operator_options_free(&options);
+        opendal_operator_options_free(options);
     }
 
-    void TearDown() override { opendal_operator_free(&this->p); }
+    void TearDown() override { opendal_operator_free(this->p); }
 };
 
 // Scenario: OpenDAL Blocking Operations
@@ -68,12 +69,12 @@ TEST_F(OpendalBddTest, FeatureTest)
     // The blocking file "test" entry mode must be file
     opendal_result_stat s = opendal_operator_stat(this->p, this->path.c_str());
     EXPECT_EQ(s.code, OPENDAL_OK);
-    opendal_metadata meta = s.meta;
-    EXPECT_TRUE(opendal_metadata_is_file(&meta));
+    opendal_metadata *meta = s.meta;
+    EXPECT_TRUE(opendal_metadata_is_file(meta));
 
     // The blocking file "test" content length must be 13
-    EXPECT_EQ(opendal_metadata_content_length(&meta), 13);
-    opendal_metadata_free(&meta);
+    EXPECT_EQ(opendal_metadata_content_length(meta), 13);
+    opendal_metadata_free(meta);
 
     // The blocking file "test" must have content "Hello, World!"
     struct opendal_result_read r = opendal_operator_blocking_read(this->p, this->path.c_str());
