@@ -648,6 +648,11 @@ impl<R: oio::Write> oio::Write for RetryWrapper<R> {
         }
     }
 
+    /// Sink will move the input stream, so we can't retry it.
+    async fn sink(&mut self, size: u64, s: oio::Streamer) -> Result<()> {
+        self.inner.sink(size, s).await
+    }
+
     async fn abort(&mut self) -> Result<()> {
         let mut backoff = self.builder.build();
 
@@ -860,6 +865,7 @@ mod tests {
         fn info(&self) -> AccessorInfo {
             let mut am = AccessorInfo::default();
             am.set_capability(Capability {
+                read: true,
                 list: true,
                 list_with_delimiter_slash: true,
                 list_without_delimiter: true,
