@@ -83,6 +83,14 @@ impl<S: Adapter> Accessor for Backend<S> {
             cap.delete = true;
         }
 
+        if cap.read && cap.write {
+            cap.copy = true;
+        }
+
+        if cap.read && cap.write && cap.delete {
+            cap.rename = true;
+        }
+
         if cap.list {
             cap.list_without_delimiter = true;
         }
@@ -230,6 +238,68 @@ impl<S: Adapter> Accessor for Backend<S> {
 
         Ok((RpList::default(), pager))
     }
+
+    async fn rename(&self, from: &str, to: &str, _: OpRename) -> Result<RpRename> {
+        let from = build_abs_path(&self.root, from);
+        let to = build_abs_path(&self.root, to);
+
+        let bs = match self.kv.get(&from).await? {
+            // TODO: we can reuse the metadata in value to build content range.
+            Some(bs) => bs,
+            None => return Err(Error::new(ErrorKind::NotFound, "kv doesn't have this path")),
+        };
+
+        self.kv.set(&to, &bs).await?;
+
+        self.kv.delete(&from).await?;
+        Ok(RpRename::default())
+    }
+
+    fn blocking_rename(&self, from: &str, to: &str, _: OpRename) -> Result<RpRename> {
+        let from = build_abs_path(&self.root, from);
+        let to = build_abs_path(&self.root, to);
+
+        let bs = match self.kv.blocking_get(&from)? {
+            // TODO: we can reuse the metadata in value to build content range.
+            Some(bs) => bs,
+            None => return Err(Error::new(ErrorKind::NotFound, "kv doesn't have this path")),
+        };
+
+        self.kv.blocking_set(&to, &bs)?;
+
+        self.kv.blocking_delete(&from)?;
+        Ok(RpRename::default())
+    }
+
+    async fn copy(&self, from: &str, to: &str, _: OpCopy) -> Result<RpCopy> {
+        let from = build_abs_path(&self.root, from);
+        let to = build_abs_path(&self.root, to);
+
+        let bs = match self.kv.get(&from).await? {
+            // TODO: we can reuse the metadata in value to build content range.
+            Some(bs) => bs,
+            None => return Err(Error::new(ErrorKind::NotFound, "kv doesn't have this path")),
+        };
+
+        self.kv.set(&to, &bs).await?;
+
+        Ok(RpCopy::default())
+    }
+
+    fn blocking_copy(&self, from: &str, to: &str, _: OpCopy) -> Result<RpCopy> {
+        let from = build_abs_path(&self.root, from);
+        let to = build_abs_path(&self.root, to);
+
+        let bs = match self.kv.blocking_get(&from)? {
+            // TODO: we can reuse the metadata in value to build content range.
+            Some(bs) => bs,
+            None => return Err(Error::new(ErrorKind::NotFound, "kv doesn't have this path")),
+        };
+
+        self.kv.blocking_set(&to, &bs)?;
+
+        Ok(RpCopy::default())
+    }
 }
 
 impl<S> Backend<S>
@@ -325,6 +395,13 @@ impl<S: Adapter> oio::Write for KvWriter<S> {
         Ok(())
     }
 
+    async fn sink(&mut self, _size: u64, _s: oio::Streamer) -> Result<()> {
+        Err(Error::new(
+            ErrorKind::Unsupported,
+            "Write::sink is not supported",
+        ))
+    }
+
     async fn abort(&mut self) -> Result<()> {
         Err(Error::new(
             ErrorKind::Unsupported,
