@@ -27,6 +27,7 @@ use http::Response;
 use super::body::IncomingAsyncBody;
 use super::parse_content_length;
 use super::AsyncBody;
+use crate::raw::oio::into_stream;
 use crate::Error;
 use crate::ErrorKind;
 use crate::Result;
@@ -76,7 +77,9 @@ impl HttpClient {
 
     /// Send a request in async way.
     pub async fn send(&self, req: Request<AsyncBody>) -> Result<Response<IncomingAsyncBody>> {
-        let url = req.uri().to_string();
+        // Uri stores all string alike data in `Bytes` which means
+        // the clone here is cheap.
+        let uri = req.uri().clone();
         let is_head = req.method() == http::Method::HEAD;
 
         let (parts, body) = req.into_parts();
@@ -85,7 +88,7 @@ impl HttpClient {
             .client
             .request(
                 parts.method,
-                reqwest::Url::from_str(&url).expect("input request url must be valid"),
+                reqwest::Url::from_str(&uri.to_string()).expect("input request url must be valid"),
             )
             .version(parts.version)
             .headers(parts.headers);
@@ -93,6 +96,7 @@ impl HttpClient {
         req_builder = match body {
             AsyncBody::Empty => req_builder.body(reqwest::Body::from("")),
             AsyncBody::Bytes(bs) => req_builder.body(reqwest::Body::from(bs)),
+            AsyncBody::Stream(s) => req_builder.body(reqwest::Body::wrap_stream(s)),
         };
 
         let mut resp = req_builder.send().await.map_err(|err| {
@@ -111,7 +115,7 @@ impl HttpClient {
 
             let mut oerr = Error::new(ErrorKind::Unexpected, "send async request")
                 .with_operation("http_util::Client::send_async")
-                .with_context("url", &url)
+                .with_context("url", uri.to_string())
                 .set_source(err);
             if is_temporary {
                 oerr = oerr.set_temporary();
@@ -130,7 +134,10 @@ impl HttpClient {
 
         let mut hr = Response::builder()
             .version(resp.version())
-            .status(resp.status());
+            .status(resp.status())
+            // Insert uri into response extension so that we can fetch
+            // it later.
+            .extension(uri.clone());
         // Swap headers directly instead of copy the entire map.
         mem::swap(hr.headers_mut().unwrap(), resp.headers_mut());
 
@@ -139,11 +146,14 @@ impl HttpClient {
             // it to interrupt so we can retry it.
             Error::new(ErrorKind::Unexpected, "read data from http stream")
                 .map(|v| if err.is_body() { v.set_temporary() } else { v })
-                .with_context("url", &url)
+                .with_context("url", uri.to_string())
                 .set_source(err)
         });
 
-        let body = IncomingAsyncBody::new(Box::new(stream), content_length);
+        let body = IncomingAsyncBody::new(
+            Box::new(into_stream::from_futures_stream(stream)),
+            content_length,
+        );
 
         let resp = hr.body(body).expect("response must build succeed");
 
