@@ -405,6 +405,8 @@ impl Accessor for GcsBackend {
             .set_root(&self.core.root)
             .set_name(&self.core.bucket)
             .set_capability(Capability {
+                create_dir: true,
+
                 stat: true,
                 stat_with_if_match: true,
                 stat_with_if_none_match: true,
@@ -427,6 +429,8 @@ impl Accessor for GcsBackend {
                 list_with_delimiter_slash: true,
                 list_without_delimiter: true,
 
+                batch: true,
+                batch_max_operations: Some(100),
                 presign: true,
                 presign_stat: true,
                 presign_read: true,
@@ -557,6 +561,65 @@ impl Accessor for GcsBackend {
         ))
     }
 
+    async fn batch(&self, args: OpBatch) -> Result<RpBatch> {
+        let ops = args.into_operation();
+        if ops.len() > 100 {
+            return Err(Error::new(
+                ErrorKind::Unsupported,
+                "gcs services only allow delete less than 100 keys at once",
+            )
+            .with_context("length", ops.len().to_string()));
+        }
+
+        let paths: Vec<String> = ops.into_iter().map(|(p, _)| p).collect();
+        let resp = self.core.gcs_delete_objects(paths.clone()).await?;
+
+        let status = resp.status();
+
+        if let StatusCode::OK = status {
+            let content_type = parse_content_type(resp.headers())?.ok_or_else(|| {
+                Error::new(
+                    ErrorKind::Unexpected,
+                    "gcs batch delete response content type is empty",
+                )
+            })?;
+            let boundary = content_type
+                .strip_prefix("multipart/mixed; boundary=")
+                .ok_or_else(|| {
+                    Error::new(
+                        ErrorKind::Unexpected,
+                        "gcs batch delete response content type is not multipart/mixed",
+                    )
+                })?
+                .trim_matches('"');
+            let multipart: Multipart<MixedPart> = Multipart::new()
+                .with_boundary(boundary)
+                .parse(resp.into_body().bytes().await?)?;
+            let parts = multipart.into_parts();
+
+            let mut batched_result = Vec::with_capacity(parts.len());
+
+            for (i, part) in parts.into_iter().enumerate() {
+                let resp = part.into_response();
+                // TODO: maybe we can take it directly?
+                let path = paths[i].clone();
+
+                // deleting not existing objects is ok
+                if resp.status().is_success() || resp.status() == StatusCode::NOT_FOUND {
+                    batched_result.push((path, Ok(RpDelete::default().into())));
+                } else {
+                    batched_result.push((path, Err(parse_error(resp).await?)));
+                }
+            }
+
+            Ok(RpBatch::new(batched_result))
+        } else {
+            // If the overall request isn't formatted correctly and Cloud Storage is unable to parse it into sub-requests, you receive a 400 error.
+            // Otherwise, Cloud Storage returns a 200 status code, even if some or all of the sub-requests fail.
+            Err(parse_error(resp).await?)
+        }
+    }
+
     async fn presign(&self, path: &str, args: OpPresign) -> Result<RpPresign> {
         // We will not send this request out, just for signing.
         let mut req = match args.operation() {
